/**
 * Copyright 2011 J Adam Crain (jadamcrain@gmail.com)
 *
 * Licensed to J Adam Crain under one or more contributor license agreements.
 * See the NOTICE file distributed with this work for additional information
 * regarding copyright ownership. J Adam Crain licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.automatak

import java.util.concurrent.{ ScheduledFuture, TimeUnit, ScheduledExecutorService, Executor }

package object executor4s {

  private class FunRun(fun: => Unit) extends Runnable {
    def run() = fun
  }

  class LongTimeConverter(count: Long) {
    def nanoseconds = NanoSeconds(count)
    def microseconds = MicroSeconds(count)
    def milliseconds = MilliSeconds(count)
    def seconds = Seconds(count)
    def minutes = Minutes(count)
    def hours = Hours(count)
    def days = Days(count)
  }

  implicit def convertLongToLongTimeConverter(count: Long) = new LongTimeConverter(count)

  def attempt[A](fun: => A): Result[A] = Result(fun)

  def dispatch(fun: => Unit)(implicit exe: Executor): Unit = exe.execute(new FunRun(fun))

  def scheduleAtFixedRate(initialDelay: Long, period: Long, unit: TimeUnit)(fun: => Unit)(implicit exe: ScheduledExecutorService): ScheduledFuture[_] = {
    exe.scheduleAtFixedRate(new FunRun(fun), initialDelay, period, unit)
  }

  def scheduleWithFixedDelay(initialDelay: Long, delay: Long, unit: TimeUnit)(fun: => Unit)(implicit exe: ScheduledExecutorService): ScheduledFuture[_] = {
    exe.scheduleWithFixedDelay(new FunRun(fun), initialDelay, delay, unit)
  }

  def schedule(delay: Long, unit: TimeUnit)(fun: => Unit)(implicit exe: ScheduledExecutorService): ScheduledFuture[_] = {
    exe.schedule(new FunRun(fun), delay, unit)
  }
}